//
//  ThirdLayoutExampleView.swift
//  LayoutProject
//
//  Created by Karin Prater on 17.10.23.
//

import SwiftUI

@available(iOS 16.0, macOS 13.0, tvOS 16.0, watchOS 9.0, *)
struct PriorityWidthHStack: Layout {
    
    let spacing: CGFloat
    var alignment: Alignment = .center
    
    func sizeThatFits(proposal: ProposedViewSize,
                      subviews: Subviews,
                      cache: inout Void) -> CGSize {
        // Return a size.
        guard !subviews.isEmpty else { return .zero }
        
      //  return proposal.replacingUnspecifiedDimensions()
        return fullIntrinsicSize(subviews: subviews, for: proposal)
    }
    
    func placeSubviews(in bounds: CGRect,
                       proposal: ProposedViewSize,
                       subviews: Subviews,
                       cache: inout Void) {
        // Place child views
        guard !subviews.isEmpty else { return }
        guard let containerWidth = proposal.width,
              let containerHeight = proposal.height else { return }
        
        let totalSpacing = totalSpacing(subviews: subviews)
        let availableWidth = containerWidth - totalSpacing
 
        var x = bounds.minX // bounds is like uikit frame with origin
  
        for (index, subView) in subviews.enumerated() {
            
            let viewWidth: CGFloat = subView.priority * availableWidth
            let viewSize = subView.sizeThatFits(.init(width: viewWidth, height: bounds.height))
            let placementProposal = ProposedViewSize(width: viewWidth,
                                                    height: containerHeight)
            let anchorPoint = anchorPoint(in: bounds,
                                          availableViewWidth: viewWidth,
                                          intrinsicViewSize: viewSize,
                                          x: &x)
            subviews[index].place(
                at: anchorPoint,
                anchor: .center,
                proposal: placementProposal)
            x += viewWidth + spacing
        }
    }
    
    private func maxSize(subviews: Subviews, for proposal: ProposedViewSize) -> CGSize {
        // unspecified: ideal size
        let availableSize = (proposal.width ?? 0) - totalSpacing(subviews: subviews)
        
        let subviewSizes = subviews.map { $0.sizeThatFits(.init(width: availableSize * $0.priority,
                                                                height: proposal.height ?? 0)) }
        let maxSize: CGSize = subviewSizes.reduce(.zero) { currentMax, subviewSize in
            CGSize(
                width: max(currentMax.width, subviewSize.width),
                height: max(currentMax.height, subviewSize.height))
        }

        return maxSize
    }
    
    private func totalSpacing (subviews: Subviews) -> CGFloat {
        var totalSpacing: CGFloat = 0
        if subviews.count > 1 {
            totalSpacing = spacing * CGFloat(subviews.count - 1)
        }
        return totalSpacing
    }
 
    private func fullIntrinsicSize(subviews: Subviews, for proposal: ProposedViewSize) -> CGSize {
        let maxSize = maxSize(subviews: subviews, for: proposal)
   
        return CGSize(width: proposal.width ?? 0,
                      height: maxSize.height)
    }
    
    
    func anchorPoint(in bounds: CGRect, 
                     availableViewWidth: CGFloat,
                     intrinsicViewSize: CGSize,
                     x: inout CGFloat) -> CGPoint {
        var anchorY = bounds.midY
        var anchorX = x + availableViewWidth / 2
        switch alignment {
            case .top:
                anchorY = (bounds.minY + intrinsicViewSize.height / 2)
            case .bottom:
                anchorY = (bounds.maxY - intrinsicViewSize.height / 2)
            case .leading, .leadingLastTextBaseline, .leadingFirstTextBaseline:
                anchorX = bounds.minX + intrinsicViewSize.width / 2
            case .trailing, .trailingFirstTextBaseline, .trailingLastTextBaseline:
                anchorX = x + availableViewWidth - intrinsicViewSize.width / 2
            case .topLeading:
                anchorX = x + intrinsicViewSize.width / 2
                anchorY = (bounds.minY + intrinsicViewSize.height / 2)
            case .topTrailing:
                anchorX = x + intrinsicViewSize.width / 2
                anchorY = (bounds.maxY - intrinsicViewSize.height / 2)
            case .bottomLeading:
                anchorX = x + intrinsicViewSize.width / 2
                anchorY = (bounds.maxY - intrinsicViewSize.height / 2)
            case .bottomTrailing:
                anchorX = x + availableViewWidth - intrinsicViewSize.width / 2
                anchorY = (bounds.maxY - intrinsicViewSize.height / 2)
            default: anchorY = bounds.midY
        }
        
        return CGPoint(x: anchorX, y: anchorY)
    }
}





