//
//  LayoutExampleView.swift
//  LayoutProject
//
//  Created by Karin Prater on 20.10.23.
//

import SwiftUI

struct LayoutExampleView: View {
    var body: some View {
        VStack(spacing: 30) {
            Text(/*@START_MENU_TOKEN@*/"Hello, World!"/*@END_MENU_TOKEN@*/)
            
            HStack(spacing: 10) {
                Text("First")
                    .padding()
                    .background(Color.yellow)
                Text("Second")
                    .padding()
                    .background(Color.cyan)
            }
            
            CustomHStack(spacing: 10) {
                Text("First")
                    .padding()
                    .background(Color.orange)
                Text("Second")
                    .padding()
                    .background(Color.red)
                
               // Color.red
            }
            .border(Color.black)
            
            
            HStack(alignment: .center, spacing: 10) {
                Text("first")
                    .background(Color.orange)
                
                Text("This is a very long text that will")
                
                Text("This is a very long text that will span over multiple lines and take up a lot of space")
                    .padding()
                    .border(Color.red)
                Text("last text")
                    .background(Color.green)
                   
            }
            
            CustomHStack(alignment: .top, spacing: 10) {
                Text("first")
                    .background(Color.orange)
                
                Text("This is a very long text that will")
                
                Text("This is a very long text that will span over multiple lines and take up a lot of space")
                    .padding()
                    .border(Color.red)
                Text("last text")
                    .background(Color.green)
                   
            }
            .border(Color.black)
        }
    }
}

struct CustomHStack: Layout {
    
    struct ViewSizeResult {
        let index: Int
        var size: CGSize
        var isFit: Bool
    }
    
    let alignment: VerticalAlignment
    let spacing: CGFloat
    
    init(alignment: VerticalAlignment = .center,
         spacing: CGFloat = 10) {
        self.alignment = alignment
        self.spacing = spacing
    }
    
    func sizeThatFits(proposal: ProposedViewSize, subviews: Subviews, cache: inout ()) -> CGSize {
        guard !subviews.isEmpty else { return .zero }
        
        return fullIntrinsicSize(subviews: subviews, for: proposal)
    }
    
    func placeSubviews(in bounds: CGRect, proposal: ProposedViewSize, subviews: Subviews, cache: inout ()) {
     
        guard !subviews.isEmpty else { return }
        
        var x = bounds.minX
        let viewSizes = calcualteSizes(subviews: subviews,
                                       for: proposal)
        for (index, subview) in subviews.enumerated() {
            let size = viewSizes[index]
            let anchorPoint = anchorPoint(in: bounds,
                                          intrinsicViewSize: size,
                                          x: &x)
            subview.place(at: anchorPoint,
                          anchor: .center,
                          proposal: ProposedViewSize(width: size.width,
                                                     height: size.height))
            x += size.width + spacing
            
        }
    }
    
    func totalSpacing(subviews: Subviews) -> CGFloat {
        var totalSpacing: CGFloat = 0
        if subviews.count > 0 {
            totalSpacing = spacing * CGFloat(subviews.count - 1)
        }
        return totalSpacing
    }
    
    func fullIntrinsicSize(subviews: Subviews,
                           for proposal: ProposedViewSize) -> CGSize {
        
        let totalSpacing = totalSpacing(subviews: subviews)
     
        let viewSizes = calcualteSizes(subviews: subviews,
                                       for: proposal)
        
        let height = viewSizes.max { $0.height < $1.height}?.height ?? 0
        let width = viewSizes.reduce(0) { $0 + $1.width}
        
        return CGSize(width: width + totalSpacing, height: height)
    }
    
    func calcualteSizes(subviews: Subviews,
                        for proposal: ProposedViewSize) -> [CGSize] {
        let availableWidth = proposal.width ?? 0 - totalSpacing(subviews: subviews)
        let individualViewOffer = availableWidth / CGFloat(subviews.count)
        
        var viewSizes = [ViewSizeResult]()
        for (index, subview) in subviews.enumerated() {
        
           let maxSize = subview.dimensions(in: .infinity)
           let idealSize = subview.dimensions(in: .unspecified)
           let minSize = subview.dimensions(in: .zero)
           
            let sizeThatFits = subview.sizeThatFits(.init(width: individualViewOffer,
                                                          height: proposal.height))
            print("... \(minSize.width) - \(idealSize.width) - \(maxSize.width) --> \(sizeThatFits.width)")
            
            let doesFit = idealSize.width == sizeThatFits.width
            
            viewSizes.append(ViewSizeResult(index: index,
                                            size: doesFit ? sizeThatFits : .zero,
                                            isFit: doesFit))
        }
        
        let widthThatIsUsedByFitViews = viewSizes.reduce(0) { $0 + $1.size.width }
        let remainingWidth = availableWidth - widthThatIsUsedByFitViews
        let countRemainingViews = viewSizes.filter { !$0.isFit }.count
        let remainingOffering = remainingWidth / CGFloat(countRemainingViews)
        
        for viewSize in viewSizes {
            if !viewSize.isFit {
             let size = subviews[viewSize.index].sizeThatFits(.init(width: remainingOffering, height: proposal.height))
                viewSizes[viewSize.index].size = size
            }
        }
        
        return viewSizes.map { $0.size }
    }
    
    func anchorPoint(in bounds: CGRect,
                     intrinsicViewSize: CGSize,
                     x: inout CGFloat) -> CGPoint {
        var anchorY = bounds.midY
        var anchorX = x + intrinsicViewSize.width / 2
        switch alignment {
            case .top:
                anchorY = (bounds.minY + intrinsicViewSize.height / 2)
            case .bottom:
                anchorY = (bounds.maxY - intrinsicViewSize.height / 2)
    
            default: anchorY = bounds.midY
        }
        
        return CGPoint(x: anchorX, y: anchorY)
    }
}


#Preview {
    LayoutExampleView()
}
