//
//  AsyncImageExampleView.swift
//  LayoutProject
//
//  Created by Karin Prater on 11.09.2023.
//

import SwiftUI

struct AsyncImageExampleView: View {
    
    let baseURLString = "https://picsum.photos/id/12/"
    
    @Environment(\.displayScale) var scale
    let size: CGFloat = 200
    var urlString: String {
        baseURLString + "\(Int(size * scale))"
    }
    
    var body: some View {
        VStack {
            AsyncImage(url: URL(string: "https://picsum.photos/id/12/200"))
            
            AsyncImage(url: URL(string: urlString)) { image in
                image.resizable()
            } placeholder: {
                ProgressView()
            }
            .frame(width: size, height: size)

            AsyncImage(url: URL(string: urlString),
                       scale: 3, 
                       transaction: .init(animation: .bouncy)) { phase in
                switch phase {
                    case .empty:
                        ZStack {
                            Color.gray
                            ProgressView()
                        }
                    case .success(let image):
                        image.resizable()
                    case .failure(let error):
                        Text(error.localizedDescription)
                        // use placeholder for production app
                    @unknown default:
                        fatalError()
                }
            }
            .frame(width: size, height: size)
            
        }
    }
}

fileprivate struct SizedAsyncImageExampleView: View {
    
    let baseURLString = "https://picsum.photos/id/12/"
    @Environment(\.displayScale) var scale
    
    var body: some View {
        GeometryReader(content: { geometry in
            AsyncImage(url: url(in: geometry.size.width),
                       scale: 3,
                       transaction: .init(animation: .bouncy)) { phase in
                switch phase {
                    case .empty:
                        ZStack {
                            Color.gray
                            ProgressView()
                        }
                    case .success(let image):
                        image.resizable()
                    case .failure(let error):
                        Text(error.localizedDescription)
                        // use placeholder for production app
                    @unknown default:
                        fatalError()
                }
            }
            .frame(width: geometry.size.width,
                   height: geometry.size.width)
        })
    }
    
    func url(in width: CGFloat) -> URL? {
        let imageWidth = width * scale
        let urlString = baseURLString + "\(Int(imageWidth))"
        return URL(string: urlString)
    }
}

fileprivate struct AspectRatioSizedAsyncImageExampleView: View {
    
    let photo = PicsumPhoto.example()
    let baseURLString = "https://picsum.photos/id/"
    @Environment(\.displayScale) var scale
    
    var body: some View {
        GeometryReader(content: { geometry in
            AsyncImage(url: url(in: geometry.size.width),
                       scale: 3,
                       transaction: .init(animation: .bouncy)) { phase in
                switch phase {
                    case .empty:
                        ZStack {
                            Color.gray
                            ProgressView()
                        }
                    case .success(let image):
                        image.resizable()
                    case .failure(let error):
                        Text(error.localizedDescription)
                        // use placeholder for production app
                    @unknown default:
                        fatalError()
                }
            }
            .frame(width: geometry.size.width,
                   height: geometry.size.width / photo.aspectRatio)
        })
    }
    
    func url(in width: CGFloat) -> URL? {
        let imageWidth = width * scale
        let imageHeight = imageWidth / photo.aspectRatio
        let urlString = "\(baseURLString)\(photo.id)/\(Int(imageWidth))/\(Int(imageHeight))"
        return URL(string: urlString)
    }
}


#Preview {
    AsyncImageExampleView()
}

#Preview("scale to fit") {
    SizedAsyncImageExampleView()
}

#Preview("Keep aspect ratio") {
    AspectRatioSizedAsyncImageExampleView()
}
