//
//  NavigationListView.swift
//  LayoutProject
//
//  Created by Karin Prater on 04.02.24.
//

import SwiftUI

struct NavigationListView: View {
    
    enum DisplayStyle: CaseIterable, Identifiable {
        case grid
        case table
        case list
        
        var icon: String {
            switch self {
                case .grid: return "square.grid.2x2"
                case .table: return "tablecells"
                case .list: return "list.bullet"
            }
        }
        var id: Self { return self }
    }
    
    @State private var folders = Folder.examples()
    @State private var folderSelection: FolderSelectionType? = nil
    @State private var selectedNote: Note? = nil
    @State private var searchTerm: String = ""
    
    @State private var displayStyle = DisplayStyle.table
    
    var body: some View {
        NavigationSplitView {
            FolderSelectionListView(folders: $folders,
                                    selection: $folderSelection.animation())
            .navigationTitle("Folders")
            .toolbar(content: {
                Button {
                    
                } label: {
                    Label("New Folder", systemImage: "folder.badge.plus")
                }

            })
            .searchable(text: $searchTerm)
            .toolbar(content: {
                ToolbarItem(placement: .secondaryAction) {
                    Picker("", selection: $displayStyle) {
                        ForEach(DisplayStyle.allCases) {
                            Image(systemName: $0.icon)
                        }
                    }
                    .pickerStyle(.segmented)
                }
            })
            
        } content: {
            
            if let folderSelection {
                Group {
                    switch displayStyle {
                        case .grid:
                            NoteSelectionGridView(notes: notes(),
                                                  selectedNote: $selectedNote)
                        case .table:
                            NoteSelectionTableView(notes: notes(), selectedNote: $selectedNote)
                        case .list:
                            NoteSelectionListView(notes: notes(),
                                                  selectedNote: $selectedNote)
                    }
                }
               
                .navigationTitle(folderSelection.displayName)
                #if os(iOS)
                .navigationBarTitleDisplayMode(.inline)
                #endif
            } else {
                ContentUnavailableView("Please select a folder", systemImage: "folder")
            }
            
            /*
             // this will not animate the note list changes nicely
             // instead will use different list instances
            switch folderSelection {
                case .all:
                    NoteSelectionListView(notes: allNotes,
                                          selectedNote: $selectedNote)
                case .favourites:
                    NoteSelectionListView(notes: favoriteNotes,
                                          selectedNote: $selectedNote)
                case .folder(let folder):
                    NoteSelectionListView(notes: folder.notes,
                                          selectedNote: $selectedNote)
                case nil:
                    ContentUnavailableView("Please select a folder", systemImage: "folder")
            }
             */
            
        } detail: {
            
            if let selectedNote {
                NoteEditDetailView(note: selectedNote)
            } else {
                ContentUnavailableView("Please select a note", systemImage: "note")
            }
        }
    }
    
    func notes() -> [Note] {
        guard searchTerm.isEmpty else {
            return notesForSearchTerm
        }
        
        switch folderSelection {
            case .all:
                return allNotes
            case .favourites:
                return favoriteNotes
            case .folder(let folder):
                return folder.notes
            case nil:
                return []
        }
    }
    
    var allNotes: [Note] {
        folders.flatMap { folder in
            folder.notes
        }
    }
    var favoriteNotes: [Note] {
        folders.flatMap { folder in
            folder.notes.filter { $0.isFavorite }
        }
    }
    
    var notesForSearchTerm: [Note] {
        folders.flatMap { folder in
            folder.notes.filter { $0.title.contains(searchTerm) }
        }
    }
}

#Preview {
    NavigationListView()
}
