//
//  PicsumPhotoLoader.swift
//  LayoutProject
//
//  Created by Karin Prater on 30.08.2023.
//

import Foundation

class PicsumPhotoLoader: ObservableObject {
    
    @Published var photos: [PicsumPhoto] = [] {
        didSet {
           reorder()
        }
    }
    @Published var photoRows: [[PicsumPhoto]] = []
    
    let page: Int
    let photosPerPage: Int
    
    init(page: Int = 1, photosPerPage: Int = 10) {
        self.page = page
        self.photosPerPage = photosPerPage
        
        loadImage()
    }
    
    func loadImage() {
        let urlString = "https://picsum.photos/v2/list?page=\(page)&limit=\(photosPerPage)"
        guard let url = URL(string: urlString) else { return }
        
        URLSession.shared.dataTask(with: url) { data, response, error in
            if let data,
                let photos = try? JSONDecoder().decode([PicsumPhoto].self, from: data){
                DispatchQueue.main.async {
                    self.photos = photos
                }
            }
        }.resume()
    }
    
    func reorder() {
        guard photos.count > 0 else { return }
        
        // Sort the photos based on aspect ratio
        let sortedPhotos = photos //.sorted { $0.aspectRatio > $1.aspectRatio }
        
        // Split the sorted photos into rows
        var rows: [[PicsumPhoto]] = []
        var currentRow: [PicsumPhoto] = []
        var currentRowAspectRatio: CGFloat = 0
        
        var singleLimit: CGFloat = 1.7
        var multiLimit: CGFloat = 3.2
        
        for photo in sortedPhotos {
            let photoAspectRatio = photo.aspectRatio
            
            // Check if the aspect ratio is larger than 1.5
            if photoAspectRatio > singleLimit {
                // Start a new row with only this photo
                rows.append([photo])
                singleLimit += 1
                multiLimit -= 0.6
            } else {
                // Check if adding this photo exceeds the aspect ratio limit for the current row
                if currentRowAspectRatio + photoAspectRatio > multiLimit {
                    // Start a new row
                    rows.append(currentRow)
                    currentRow = []
                    currentRowAspectRatio = 0
                    
                    singleLimit -= 1
                    multiLimit += 0.6
                }
                
                // Add the photo to the current row
                currentRow.append(photo)
                currentRowAspectRatio += photoAspectRatio
            }
        }
        rows.append(currentRow)
        
        self.photoRows = rows
    }
    
    
    
    
    static func preview() -> PicsumPhotoLoader {
        let loader = PicsumPhotoLoader()
        loader.photos = [PicsumPhoto.example(), PicsumPhoto.example2(), PicsumPhoto.example3(), PicsumPhoto.example4()]
        return loader
    }
}
